use std::collections::HashMap;

use std::time::Instant;

pub struct GameTimer {
    pub(crate) game_start: Instant,
    pub(crate) last_frame: Instant,
    pub(crate) stopwatches: HashMap<String, Instant>,
}

impl GameTimer {
    pub fn new() -> Self {
        Self {
            game_start: Instant::now(),
            last_frame: Instant::now(),
            stopwatches: HashMap::new(),
        }
    }

    pub fn frame_update(&mut self) {
        self.last_frame = Instant::now()
    }

    pub fn delta_t(&self) -> f32 {
        Instant::now().duration_since(self.last_frame).as_secs_f32()
    }

    pub fn game_time_passed(&self) -> f32 {
        Instant::now().duration_since(self.game_start).as_secs_f32()
    }

    pub fn start_stopwatch(&mut self, name: impl ToString) {
        let _ = self.stopwatches.insert(name.to_string(), Instant::now());
    }

    pub fn get_stopwatch_time(&self, name: &str) -> Option<f32> {
        self.stopwatches
            .get(name)
            .map(|start_time| Instant::now().duration_since(*start_time).as_secs_f32())
    }

    pub fn delete_stopwatch(&mut self, name: &str) {
        let _ = self.stopwatches.remove(name);
    }
}
